#ifndef STUDENTTRACKER_H
#define STUDENTTRACKER_H

/* Header file to define the beacons message format as well as any
 * constants that might be needed 
 * 
 * @author Maria Kazandjieva, <mariakaz@cs.stanford.edu>
 * @date Sep 10 2009
 * @updated Sept 10 2009
 *
 */

#include <AM.h>

/* by default, boradcast a beacon message every X seconds */

#define DEFAULT_BEACON_INTERVAL 20480  //1 sec is 1024 ticks 

/* we will give randomness in the beacon interval
 * so the beacon interval will be 
 * DEFAULT_BEACON_INTERVAL r where r is in [-0.5* DEFAULT_INTERVAL_RANDOMNESS, 0.5* DEFAULT_INTERVAL_RANDOMNESS]
 * */
#define DEFAULT_INTERVAL_RANDOMNESS 1024

/* have a default RSSI threshold for logging packets */
#define DEFAULT_RSSI_THRESHOLD 221  //221 corresponds to -80 dbm

/* number of packets stored in the buffer before writing to flash */
#define NREADINGS 20

/* AM ID for the communucations */
#define STUDENT_AM_ID 171
enum {
   AM_CONTROL_PACKET_T = 172, /* for mig only. Needs to be same as STUDENT_AM_ID */
   AM_DATA_T = 173,
};

/* where to start reading the configuration info on flash */
#define CONFIG_ADDR 0

/* number of fake data packets to send for testing purposes */
#define NUM_TEST_PACKETS 10

/* the structure to be included in broadcast packets */
typedef nx_struct beacon_msg_t {
    nx_uint8_t type;
    nx_am_addr_t source;
    nx_uint16_t seqno;
} beacon_msg_t;

/* this is the structure of thte flash log entries */
typedef nx_struct log_entry_t {
    nx_am_addr_t source;
    nx_uint16_t seqno;
    nx_uint8_t rssi;
    nx_uint16_t my_seqno;
} log_entry_t;

/* configuration structure that will be written to the config volume*/
typedef struct config_t {
    bool use;
    uint8_t should_beacon;  // 1 = TRUE; 0 = FALSE;
    uint16_t beacon_interval;
    uint8_t rssi_threshold;
    uint8_t pingable; //1 = TRUE; 0 = FALSE;
    bool active; // opted in or out
    uint16_t seqno;
} config_t;


/* struct control packets of type PING, PING_REPLY, RETRIEVE, END */
typedef nx_struct control_packet_t {
    nx_uint8_t type;    // valid for all control packets
    nx_am_addr_t sourceid;  // valid for ping response packet
    nx_am_addr_t startid;   // valid for ping packet
    nx_am_addr_t endid;     // valid for ping packet
} control_packet_t;

/* format of packets that send the log data to the base station
 * packet type is DATA
 */
typedef nx_struct data_t {
    nx_uint8_t type;
    nx_uint8_t num_entries;
    log_entry_t entries[NREADINGS];
    //log_entry_t e;
} data_t;

/* types of network packets */
enum {
    BEACON = 1,
    PING_REQUEST = 2,
    PING_REPLY = 3,
    RETRIEVE = 4,
    DATA = 5,
    END = 6,
    TEST = 7,
};

#endif
